/* リスト3.20 AR.Droneをキーボードで操縦するプログラム */

import com.shigeodayo.ardrone.processing.*;

ARDroneForP5 ardrone;
boolean      isFlying = false; // 飛行状態を表すフラグ
int          speed = 20;       // スピード(1～100)

void setup() {
  size(320, 240);

  // AR.Droneの接続設定
  ardrone = new ARDroneForP5();
  ardrone.connect();
  ardrone.connectNav();
  ardrone.connectVideo();
  ardrone.start();
}

void draw() {
  // AR.Droneからカメラ画像を取得して表示
  PImage img = ardrone.getVideoImage(false);
  if (img==null) return;
  image(img, 0, 0, width, height);

  // センサ情報を取得
  float roll = ardrone.getRoll();               // ロール角
  float pitch = ardrone.getPitch();             // ピッチ角
  float yaw = ardrone.getYaw();                 // ヨー角
  float altitude = ardrone.getAltitude();       // 高度
  float[] velocity = ardrone.getVelocity();     // 速度
  int battery = ardrone.getBatteryPercentage(); // バッテリ残量

  // センサ情報を画面に表示
  text("roll:"+roll+"\npitch:"+pitch+"\nyaw:"+yaw, 20, 85);
  text("altitude:"+altitude, 20, 127);
  text("vx:"+velocity[0]+"\nvy:"+velocity[1], 20, 140);
  text("battery:"+battery+"%", 20, 170);
}

// キー入力による操作
void keyPressed() { 
  if      (key=='1') { ardrone.setHorizontalCamera(); } // 前カメラ
  else if (key=='2') { ardrone.setVerticalCamera(); }   // 下カメラ
  else if (key=='w') { ardrone.forward(speed); }   // 前  
  else if (key=='s') { ardrone.backward(speed); }  // 後
  else if (key=='a') { ardrone.goLeft(speed); }    // 左
  else if (key=='d') { ardrone.goRight(speed); }   // 右
  else if (key=='e') { ardrone.spinRight(speed); } // 右回転
  else if (key=='q') { ardrone.spinLeft(speed); }  // 左回転
  else if (key=='x') { ardrone.up(speed); }        // 上昇
  else if (key=='c') { ardrone.down(speed); }      // 下降
  else if (key==' ') {
    if (isFlying) ardrone.landing(); // 着陸
    else ardrone.takeOff();          // 離陸
    isFlying = !isFlying;            // フラグを反転
  }
}

// キーを離したときに静止命令を送る
void keyReleased() {
  // カメラ切り替え命令以外の場合に限る
  if ( key!='1' && key!='2' && key!='3' && key!='4') {  
    ardrone.stop();  // 静止
  }
}
