/* リスト4.3 ARキャンドルのプログラム */

import jp.nyatla.nyar4psg.*; // ARライブラリ
import processing.video.*;   // ビデオライブラリ
import processing.serial.*;  // シリアル通信ライブラリ
import cc.arduino.*;         // Arduinoライブラリ
import picking.*;            // ピッキングライブラリ
import ppopupmenu.*;         // ポップアップメニューライブラリ

Capture      cam;      // カメラ
MultiMarker  ar;       // ARマーカ
Arduino      arduino;  // Arduino
Picker       picker;   // ピッキング
PPopupMenu   menu;     // ポップアップメニュー

// 使用するピン
int[] pinRed  = { 2, 4, 6 }; // 2色LEDの赤色の脚が接続されているピン
int[] pinBlue = { 3, 5, 7 }; // 2色LEDの青色の脚が接続されているピン

// マーカに対するLEDの相対位置 （自分で作ったものに合わせて変更してください）
float[][] offset = { {19.0, 27.0 ,32.0}, {0.0, 27.0 ,32.0},
                     {-19.0, 27.0 ,32.0}  };

// 選択されているオブジェクトのID
int selected_id = -1;

void setup() {
  // 画面サイズとキャプチャの設定
  size(800,600,P3D);
  cam = new Capture(this,width,height);
  
  // ARマーカの準備
  ar = new MultiMarker(this,width,height,"camera_para.dat",NyAR4PsgConfig.CONFIG_PSG);
  ar.addNyIdMarker(0,30); // 30mmのIDマーカを登録（NyId=0）

  // Arduinoの準備（ポート番号は自分の環境に合わせて設定してください）
  arduino = new Arduino(this, Arduino.list()[0], 57600);
  for (int i=0; i<3; i++) {
    arduino.pinMode( pinRed[i],  Arduino.OUTPUT );
    arduino.pinMode( pinBlue[i], Arduino.OUTPUT );
  }

  // メニューの準備
  menu = new PPopupMenu(this);
  menu.addMenuItem("red", "setRed");       //「red」のメニュー
  menu.addMenuItem("blue", "setBlue");     //「blue」のメニュー
  menu.addMenuItem("purple", "setPurple"); //「purple」のメニュー
  menu.addSeparator();                     // 区切り線
  menu.addMenuItem("off", "turnOff");      //「off」のメニュー

  // ピッキングの準備
  picker = new Picker(this);
}

void draw() {
  if (cam.available()==false) return; // カメラの用意ができていないときは処理をしない
  cam.read();                         // カメラ画像の読み込み  
  background(0);                      // 画面の初期化
  ar.drawBackground(cam);             // 背景画像の描画
  ar.detect(cam);                     // マーカ認識

  // マーカが認識されていたらピッキング用のオブジェクトを描画する
  if ( ar.isExistMarker(0) ) {
    ar.beginTransform(0);
      for (int i=0; i<3; i++) {
        picker.start(i); // ピッキング用のIDを付与
        pushMatrix();

          // 選択されていたら枠線を描画する
          if ( i==selected_id ) { stroke(0,0,0); }
          else { noStroke(); }

          // オブジェクトの色を無色透明にする          
          fill(0,0,0,0);

          // LEDのある場所までずらして直方体を描画
          translate( offset[i][0], offset[i][1], offset[i][2] );
          box(5,5,18);

        popMatrix();
      }
    ar.endTransform();
    picker.stop();
  }

  // マウスカーソルの下にあるオブジェクトを判定
  selected_id = picker.get(mouseX, mouseY);
}

// オブジェクトの上で右クリックされたらメニューを表示
void mouseClicked() {
  if ( mouseButton == RIGHT && selected_id >= 0 ) {
    menu.show();
  }
}

// 選択されているLEDを赤色にする
void setRed() {
  arduino.digitalWrite( pinRed[selected_id],  Arduino.HIGH );
  arduino.digitalWrite( pinBlue[selected_id], Arduino.LOW );
}

// 選択されているLEDを青色にする
void setBlue() {
  arduino.digitalWrite( pinRed[selected_id],  Arduino.LOW );
  arduino.digitalWrite( pinBlue[selected_id], Arduino.HIGH );
}

// 選択されているLEDを紫色にする
void setPurple() {
  arduino.digitalWrite( pinRed[selected_id],  Arduino.HIGH );
  arduino.digitalWrite( pinBlue[selected_id], Arduino.HIGH );
}

// 選択されているLEDをOFFにする
void turnOff() {
  arduino.digitalWrite( pinRed[selected_id],  Arduino.LOW );
  arduino.digitalWrite( pinBlue[selected_id], Arduino.LOW );
}
