/* リスト4.5 TwittARのプログラム */

import jp.nyatla.nyar4psg.*;  // ARライブラリ
import processing.video.*;    // ビデオライブラリ
import twitter4j.*;           // Twitterライブラリ

// マーカに対応付けるTwitterのユーザ名
String user_name[] = { "kougaku", "drinami", "nyatla", "narumin", "_anohito",
                       "shigeodayo", "hixi_hyi", "cmrr_xxx", "ar3bros", "tenkijp" };                   
static int NUM_MARKER  = 10;  // マーカの数
static int SIZE_MARKER = 40;  // マーカのサイズ（mm）

Twitter twitter;                                    // Twitter
Capture cam;                                        // キャプチャ
MultiMarker ar;                                     // ARマーカ処理
PImage[] img_icon = new PImage [NUM_MARKER];        // Twitterのユーザアイコン画像
PImage img_arrow1 = null;                           // 矢印画像（一方向フォロー）
PImage img_arrow2 = null;                           // 矢印画像（相互フォロー）
PImage img_loading = null;                          // ロード中画像
boolean[] isWorking = new boolean[NUM_MARKER];      // スレッドが動作中フラグ
long[] user_id = new long [NUM_MARKER];             // TwitterのユーザID
int[][] relation = new int[NUM_MARKER][NUM_MARKER]; // フォロー関係を表わすテーブル
static int UNKNOWN = 0;                             // <フォロー不明>
static int NOT_FOLLOW = 1;                          // <フォローしてない>
static int FOLLOW  = 2;                             // <フォローしている>

void setup() { 
  size(640,480,P3D);                      // 画面サイズの設定
  cam = new Capture(this, width, height); // キャプチャの準備
  
  // マーカの準備
  ar = new MultiMarker(this, width, height, "camera_para.dat", NyAR4PsgConfig.CONFIG_PSG);
  for (int id=0; id<NUM_MARKER; id++) {
    ar. addNyIdMarker(id,SIZE_MARKER);
  }

  // TwitterのユーザIDの取得
  twitter = new TwitterFactory().getInstance();
  try {
    for (int i=0; i<NUM_MARKER; i++) {
      user_id[i] = twitter.showUser(user_name[i]).getId();
    }
  } catch(TwitterException e){
    println(e.getStatusCode());
  }

  // 変数の初期化
  for (int i=0; i<NUM_MARKER; i++)
    for (int j=0; j<NUM_MARKER; j++)
      relation[i][j] = UNKNOWN;
  
  // スレッドが動作中であることを示すフラグの初期化
  for (int i=0; i<NUM_MARKER; i++)
    isWorking[i] = false;
  
  // 画像の準備
  img_arrow1 = loadImage("arrow1.gif");
  img_arrow2 = loadImage("arrow2.gif");
  img_loading = loadImage("loading.gif");

  // フォントの設定
  PFont font = loadFont("HGSoeiKakugothicUB-50.vlw");
  textFont(font);
  textSize(10);
  fill(0,0,0);
}

void draw() {
  if (cam.available()==false) return; // カメラの用意ができていないときは処理をしない
  cam.read();                         // カメラ画像の読み込み
  background(0);                      // 画面の初期化
  imageMode(CORNER);                  // 画像の原点を左上にする
  ar.drawBackground(cam);             // 背景画像の描画
  ar.detect(cam);                     // マーカ認識

  // 各マーカについての処理
  for (int i=0; i<NUM_MARKER; i++) {
    if ( ar.isExistMarker(i) ) {
      ar.beginTransform(i);
        imageMode(CENTER);
        rotate(PI);
        if ( img_icon[i] != null ) {
          // ユーザアイコンを表示
          image( img_icon[i], 0, 0, SIZE_MARKER, SIZE_MARKER );
        } else {
          // ユーザアイコンがない時はロード中画像を出し，アイコン取得スレッドを実行
          image( img_loading, 0, 0, SIZE_MARKER, SIZE_MARKER );
          GetIconThread git = new GetIconThread(i);
          git.start();
        }
        // ユーザ名の表示
        text( user_name[i], -0.5*SIZE_MARKER, 0.8*SIZE_MARKER, 1 );
      ar.endTransform();
    }
  }

  // マーカの組についての処理
  for (int i=0; i<NUM_MARKER; i++) {
    for (int j=i+1; j<NUM_MARKER; j++) {
      if ( ar.isExistMarker(i) && ar.isExistMarker(j) ) {
        // 2つのマーカに対応するユーザのフォロー関係を調べるスレッドを実行
        FollowCheckThread fct = new FollowCheckThread(i,j);
        fct.start();

        // スクリーン上のマーカの中心座標を取得する
        PVector pos1 = getCenterOfMarker(i);
        PVector pos2 = getCenterOfMarker(j);
        
        // フォローの状態によって描画内容を変える
        if ( relation[i][j] == FOLLOW && relation[j][i] == FOLLOW ) {
          drawRelation( pos1, pos2, img_arrow2, 50 ); // 相互フォロー
        } else if ( relation[i][j] == FOLLOW) {
          drawRelation( pos1, pos2, img_arrow1, 50 ); // user1がuser2を一方的にフォロー
        } else if ( relation[j][i] == FOLLOW ) {
          drawRelation( pos2, pos1, img_arrow1, 50 ); // user2がuser1を一方的にフォロー
        }
      }
    }
  }
}

// 2つのベクトルの角度を計算する
float getAngleFromVector( float x1, float y1, float x2, float y2 ) {
  float cross = x2*y1 - y2*x1;
  float dot   = x2*x1 + y2*y1;
  return -atan2(cross,dot);
}

// スクリーン上のマーカの中心座標を計算する
PVector getCenterOfMarker(int id) {
  PVector[] vec = ar.getMarkerVertex2D(id);
  return new PVector( (vec[0].x + vec[2].x)/2,  (vec[0].y + vec[2].y)/2 );
}

// 矢印を描画する
void drawRelation( PVector pos1, PVector pos2, PImage arrow_image, int size ) { 
  float angle = getAngleFromVector( 1, 0, pos2.x - pos1.x, pos2.y - pos1.y );
  imageMode(CENTER);
  pushMatrix();
    translate(pos1.x, pos1.y);
    rotate( angle );
    float d = dist(pos1.x, pos1.y, pos2.x, pos2.y);
    for (int i=1; i<d/size-0.5; i++) {
      image( arrow_image, i*size, 0, size, size );
    }
  popMatrix();
}

// userId1がuserId2をフォローしているか調べる
int checkFollow( long userId1, long userId2) {
  try {
    // userId1がフォローしている人のリストを取得し，その中にuserId2があるか調べる
    long[] friendsIds = twitter.getFriendsIDs(userId1,-1).getIDs();
    for (int i=0; i<friendsIds.length; i++) {
      if ( friendsIds[i]==userId2 ) return FOLLOW;
    }
  } catch(TwitterException e){
    println(e.getStatusCode());
  }
  return NOT_FOLLOW;
}

// マーカに対応するユーザのアイコン画像を取得するスレッド
class GetIconThread extends Thread {
  int i; // マーカID

  GetIconThread(int marker_id) {
    i = marker_id; // マーカIDを引数として受け取る
  }

  public void run(){
    if ( isWorking[i] ) return; // 同じマーカに対する多重処理を防止
    isWorking[i] = true;
    try {
      ProfileImage profileimage = twitter.getProfileImage( user_name[i],
                                                           ProfileImage.NORMAL);
      img_icon[i] = loadImage( profileimage.getURL().toString());
    } catch(TwitterException e) {
      e.printStackTrace();
    }
    isWorking[i] = false;
  }
}

// 2つのマーカに対応するユーザのフォロー関係を調べるスレッド
class FollowCheckThread extends Thread {
  int a, b; // 2つのマーカのID

  FollowCheckThread( int marker_id1, int marker_id2 ) {
    a = marker_id1; // マーカIDを引数として受け取る
    b = marker_id2;
  }

  public void run(){
    if ( isWorking[a] ) return; // 同じマーカに対する多重処理を防止
    isWorking[a] = true;
    if ( relation[a][b] == UNKNOWN ) relation[a][b] = checkFollow( user_id[a], user_id[b] );
    if ( relation[b][a] == UNKNOWN ) relation[b][a] = checkFollow( user_id[b], user_id[a] );  
    isWorking[a] = false;
  }
}
